'''
Script for the paper "A decade of detailed observations (2008--2018) in steep 
bedrock permafrost at Matterhorn Hörnligrat (Zermatt, CH)"

Script to perform the experiments

Copyright (c) 201, Swiss Federal Institute of Technology (ETH Zurich)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

* Neither the name of the copyright holder nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

@date: January 23, 2019
@author: Samuel Weber and Matthias Meyer

usage: manage_GSNdata.py [-h] [--gsn2raw] [--load] [--filter] [--clean]
                         [--aggregationInterval AGGREGATIONINTERVAL]
                         [--export2csv] [--sanityPlot] [--gsn2img]
                         [--nImg NIMG] [-p PATH] [-y1 YEARBEGIN]
                         [-m1 MONTHBEGIN] [-d1 DAYBEGIN] [-y2 YEAREND]
                         [-m2 MONTHEND] [-d2 DAYEND]

optional arguments:
  -h, --help            show this help message and exit
  --gsn2raw             Get raw data from GSN and store locally (step1)
  --load                Load locally stored data (step2)
  --filter              Filter data (according reference values, step3)
  --clean               Clean data manually (step4)
  --aggregationInterval AGGREGATIONINTERVAL
                        Aggregate data over timewindows of X minutes (step5)
  --export2csv          Export data in yearly csv files (step6)
  --sanityPlot          Plot for sanity check (step7)
  --gsn2img             Downlod images from GSN and convert NEF to JPEG
  --nImg NIMG           Number of images to download and convert
  -p PATH, --path PATH  Relative path for the data output
  -y1 YEARBEGIN, --yearBegin YEARBEGIN
                        Year begin
  -m1 MONTHBEGIN, --monthBegin MONTHBEGIN
                        Month begin
  -d1 DAYBEGIN, --dayBegin DAYBEGIN
                        Day begin
  -y2 YEAREND, --yearEnd YEAREND
                        Year end
  -m2 MONTHEND, --monthEnd MONTHEND
                        Month end
  -d2 DAYEND, --dayEnd DAYEND
                        Day end
'''
def main():
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 
    ## SETUP PYTHON ENVIRONMENT
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 
    
    # Load functions
    import sys
    import socket
    import argparse
    import os as os
    import numpy as np
    import pandas as pd
    import datetime as dt
    import matplotlib as mpl
    import matplotlib.pyplot as plt
    
    # Load functions in relative path
    sys.path.append('~/permasense/')
    from permasense.GSNdata import get_GSNdata, save_data, load_data, filter_data, clean_data, get_GSNimg
    
    # For proper plotting on server
    if socket.gethostname() == 'tik42x':
        mpl.use('Agg')
     
    
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 
    ## ASSIGN ARGUMENTS TO VARIABLES
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 

    args = parseArguments()
    
    # Define path to data
    PATH_DATA = args.path
    
    # Define time range
    TBEG = dt.datetime(args.yearBegin,args.monthBegin,args.dayBegin)
    TEND = dt.datetime(args.yearEnd,args.monthEnd,args.dayEnd)
    
    # Get raw data from GSN (step1) and produce derived data product (step2 - step7)
    GSN2RAW = args.gsn2raw
    print(GSN2RAW)
    LOAD = args.load
    FILTER = args.filter
    CLEAN = args.clean
    AGGREGATION_INTERVAL = args.aggregationInterval
    EXPORT2CSV = args.export2csv
    SANITY_PLOT = args.sanityPlot
    if FILTER | CLEAN | EXPORT2CSV | SANITY_PLOT:
        LOAD = True
    
    # Get high-resolution images (.NEF) and convert to .JPG
    GSN2IMG = args.gsn2img
    N_IMG = args.nImg
    
    
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 
    ## DICTIONARIES ASSIGNING 'VIRTUAL_SENSOR' TO 'POSITION'
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 
    DEPO_VS = {
            'MH01': {'displacement', 'temperature_rock', 'temperature_fracture'},  #ok
            'MH02': {'displacement', 'temperature_rock', 'temperature_fracture'},  #ok
            'MH03': {'displacement', 'temperature_rock', 'temperature_fracture'},  #ok
            'MH04': {'displacement', 'temperature_rock', 'temperature_fracture'},  #ok
            'MH05': {'temperature_fracture', 'resistivity_fracture'},              #ok but resistivity
            'MH06': {'displacement', 'temperature_rock'},                          #ok
            'MH07': {'temperature_fracture', 'resistivity_fracture'},              #ok but resistivity
            'MH08': {'displacement', 'temperature_rock'},                          #ok
            'MH09': {'displacement', 'temperature_fracture'},                      #ok
            'MH10': {'temperature_rock', 'resistivity_rock'},                      #ok but resistivity
            'MH11': {'temperature_rock', 'resistivity_rock'},                      #ok but resistivity
            'MH12': {'temperature_rock', 'resistivity_rock'},                      #ok but resistivity
            'MH18': {'displacement'},                                              #ok
            'MH20': {'displacement'},                                              #ok
            'MH21': {'displacement'},                                              #ok
            'MH22': {'displacement'},                                              #ok
            'MH27': {'temperature_rock'},                                          #ok
            'MH30': {'temperature_rock'},                                          #ok
            'MH46': {'temperature_rock'},                                          #ok
            'MH47': {'temperature_rock'},                                          #ok
    
            'MH15': {'radiometer__conv'},                                          #ok
            'MH25': {'vaisalawxt520windpth','vaisalawxt520prec'},                  #ok
    
            'MH33': {'gps_differential__batch__daily','gps_inclinometer'},         #ok
            'MH34': {'gps_differential__batch__daily','gps_inclinometer'},         #ok
            'MH35': {'gps_differential__batch__daily','gps_inclinometer'},         #ok
            'MH40': {'gps_differential__batch__daily'},                            #ok
            'MH42': {'gps_differential__batch__daily'},                            #ok
            #'MH43': {'gps_differential__batch__daily'},
            }
    
    DEPO_IMG = {
            'MH19': {'binary__mapped'}#{'camera_pos19__scaled'}
            }
    
    
    
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 
    ## Create PATH_DATA if it does not exist yet
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 
    
    os.makedirs(PATH_DATA, exist_ok=True)
    os.makedirs(PATH_DATA + 'gnss_derived_data_products', exist_ok=True)
    os.makedirs(PATH_DATA + 'timelapse_images', exist_ok=True)
    os.makedirs(PATH_DATA + 'timeseries_data_raw', exist_ok=True)
    os.makedirs(PATH_DATA + 'timeseries_derived_data_products', exist_ok=True)
    os.makedirs(PATH_DATA + 'timeseries_sanity_plots', exist_ok=True)
    os.makedirs(PATH_DATA + 'timeseries_sanity_plots/' + 'channelwise', exist_ok=True)
    
    
    
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 
    ## GSN timeseries data
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 
    
    # Loop through keys (depo) in dictionary DEPO_VS
    for depo in DEPO_VS:
        print(depo)
        if depo[0:2] == 'MH':
            deployment = 'matterhorn'
        position = int(depo[2:4])
        
        
        # Loop through values for a given key of dictionary DEPO_VS
        for vsensor in list(DEPO_VS[depo]):
            print(vsensor)
            # Step 1: Query data from GSN server and save it locally
            if GSN2RAW:
                df = get_GSNdata(deployment, position, vsensor, TBEG, TEND)
                if len(df) > 1:
                    if vsensor == 'gps_differential__batch__daily':
                        save_data(df, PATH_DATA + 'gnss_derived_data_products/', depo, vsensor, file_type='csv')
                        save_data(df, PATH_DATA + 'gnss_derived_data_products/', depo, vsensor, yearly=True, iso=True, file_type='csv')
                    else:
                        save_data(df, PATH_DATA + 'timeseries_data_raw/', depo, vsensor, file_type='csv')
                        save_data(df, PATH_DATA + 'timeseries_data_raw/', depo, vsensor, yearly=True, iso=True, file_type='csv')
    
                print('Data from position {:s} and virtual sensor {:s} is imported (gsn query) as pd.DataFrame and stored as csv-file.'.format(depo,vsensor))
                print(df.describe())
                
        # Loop through values for a given key of dictionary DEPO_VS but without GPS
        DEPO_VS_but_DEPO_GPS = list(vs for vs in list(DEPO_VS[depo]) if vs != 'gps_differential__batch__daily')
        
       
        for vsensor in DEPO_VS_but_DEPO_GPS:
            print(vsensor)
    
            # Step 2: Load data
            if LOAD:
                if vsensor == 'gps_differential__batch__daily':
                    df = load_data(PATH_DATA + 'gnss_derived_data_products/', depo, vsensor, file_type='csv')
                else:
                    df = load_data(PATH_DATA + 'timeseries_data_raw/', depo, vsensor, file_type='csv')
            df_raw = df.copy()
    
            
            # Step 3 (optional): Filter data (according reference values)
            if FILTER:
                df = filter_data(df, depo)
            df_filt = df.copy()
            
            # Step 4 (optional): Clean data manually
            if CLEAN:
                try:
                    df = clean_data(df, depo, vsensor)
                except:
                    print('An issue with data cleaning raised - please check the data')
                    pass
            df_filt_clean = df.copy()
    
            # Step 5 (optional): Aggregate data
            if AGGREGATION_INTERVAL > 0:
                try:
                    if vsensor == 'vaisalawxt520prec':
                        f = {'position':'mean', 'device_id':'mean', 'rain_accumulation':'sum', 'rain_duration':'sum', 'rain_intensity':'mean', 'rain_peak_intensity':'max', 'hail_accumulation':'sum', 'hail_duration':'sum', 'hail_intensity':'mean', 'hail_peak_intensity':'mean'}
                        df = df.groupby(pd.TimeGrouper(freq=str(AGGREGATION_INTERVAL) + 'Min')).agg(f)
                    elif vsensor == 'vaisalawxt520windpth':
                        f = {'position':'mean', 'device_id':'mean', 'wind_direction_minimum':'min', 'wind_direction_average':'mean', 'wind_direction_maximum':'max', 'wind_speed_minimum':'min', 'wind_speed_average':'mean', 'wind_speed_maximum':'max', 'temp_air':'mean', 'temp_internal':'mean', 'relative_humidity':'mean', 'air_pressure':'mean'}
                        df = df.groupby(pd.TimeGrouper(freq=str(AGGREGATION_INTERVAL) + 'Min')).agg(f)
    
                    else:
                        df = df.groupby(pd.TimeGrouper(freq=str(AGGREGATION_INTERVAL) + 'Min')).aggregate(np.mean)
                        #df = df.groupby(pd.TimeGrouper(freq=str(AGGREGATION_INTERVAL) + 'Min')).aggregate(np.median)
                except:
                    print('There was an issue with time aggregation')
                    pass
            df_agg = df.copy()
                    
            # Step 6: Export to csv
            if EXPORT2CSV:
                for col in ['position', 'device_id',  'ref1',  'ref2',  'ref3',  'ref4',  'ref5',  'ref6']:
                    try:
                        df_csv = df.drop([col], axis=1, inplace=False)
                    except:
                        pass
    
                try:
                    save_data(df_csv, PATH_DATA + 'timeseries_derived_data_products/', depo, vsensor, yearly=True, iso=True, file_type='csv', float_format=4)
                except:
                    pass
                
            # Step 7: Plot for sanity check
            if SANITY_PLOT:
                for i, col in enumerate(df_agg):
                    if col not in ['device_id', 'position', 'ref1', 'ref2', 'ref3', 'ref4', 'ref5', 'ref6']:
                        print(col)
                        plt.figure(figsize=(24/2.54, 18/2.54), facecolor='w', edgecolor='k')
                        plt.plot(df_raw[col][~df_raw[col].isnull()] , color='C0')
                        plt.plot(df_filt[col], color='C1')
                        plt.plot(df_filt_clean[col], color='C2')
                        plt.plot(df_agg[col], color='C3')
                        plt.title(' DEPO: {:s} \n VSENSOR: {:s}\n VARIABLE: {:s}'.format(depo,vsensor,col), loc='left')
                        
                        diff = df_agg[col].max() - df_agg[col].min()
                        try:
                            plt.ylim([df_agg[col].min() - diff/10, df_agg[col].max() + diff/10])
                        except:
                            pass
                        plt.legend(['raw','+ filtered', '+ cleaned', '+ aggregated'])
                        print(mpl.rcParams['agg.path.chunksize'])
                        mpl.rcParams['agg.path.chunksize'] = 10000
                        plt.savefig('{:s}timeseries_sanity_plots/channelwise/{:s}_{:s}_{:s}.png'.format(PATH_DATA , depo, vsensor, col), dpi=200)
                        mpl.rcParams['agg.path.chunksize'] = 0
                
                plt.figure(figsize=(24/2.54, 18/2.54), facecolor='w', edgecolor='k')
                for i, col in enumerate(df_agg):
                    if col not in ['device_id', 'position', 'ref1', 'ref2', 'ref3', 'ref4', 'ref5', 'ref6']:
                        if ~df_agg[col].isnull().all():
                            plt.plot(df_agg[col])
                            print(col)
                plt.title(' DEPO: {:s} \n VSENSOR: {:s}\n All used series'.format(depo,vsensor), loc='left')
                plt.legend()
                plt.savefig('{:s}timeseries_sanity_plots/{:s}_{:s}_all.png'.format(PATH_DATA , depo, vsensor), dpi=300)
    
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 
    ## GSN images
    ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## 
    if GSN2IMG:
        # Loop through keys (depo) in dictionary DEPO_IMG
        for depo in DEPO_IMG:
            if depo[0:2] == 'MH':
                deployment = 'matterhorn'
            position = int(depo[2:4])
            
            # Loop through values for a given key of dictionary DEPO_IMG
            for vsensor in list(DEPO_IMG[depo]):
                print(vsensor)
                df = get_GSNimg(PATH_DATA + 'timelapse_images/', deployment, position, vsensor, TBEG, TEND, n_img =N_IMG)



def parseArguments():
    import argparse
    import datetime as dt
    
    # Create argument parser
    parser = argparse.ArgumentParser()

    # Positional (mandatory) arguments
    #parser.add_argument("positionalArgument1", help="Positional argument 1", type=float)

    # Optional arguments
    parser.add_argument("--gsn2raw", help="Get raw data from GSN and store locally (step1)", action='store_false')
    parser.add_argument("--load", help="Load locally stored data (step2)", action='store_false')
    parser.add_argument("--filter", help="Filter data (according reference values, step3)", action='store_false')
    parser.add_argument("--clean", help="Clean data manually (step4)", action='store_false')
    parser.add_argument("--aggregationInterval", help="Aggregate data over timewindows of X minutes (step5)", type=int, default=60)
    parser.add_argument("--export2csv", help="Export data in yearly csv files (step6)", action='store_false')
    parser.add_argument("--sanityPlot", help="Plot for sanity check (step7)", action='store_false')
    parser.add_argument("--gsn2img", help="Downlod images from GSN and convert NEF to JPEG", action='store_false')
    parser.add_argument("--nImg", help="Number of images to download and convert", type=int, default=10)
    parser.add_argument("-p", "--path", help="Relative path for the data output", type=str, default='./../../pangaea_mh_data_2008_2018/')
    parser.add_argument("-y1", "--yearBegin", help="Year begin", type=int, default=2008)
    parser.add_argument("-m1", "--monthBegin", help="Month begin", type=int, default=1)
    parser.add_argument("-d1", "--dayBegin", help="Day begin", type=int, default=1)
    parser.add_argument("-y2", "--yearEnd", help="Year end", type=int, default=dt.datetime.now().year)
    parser.add_argument("-m2", "--monthEnd", help="Month end", type=int, default=dt.datetime.now().month)
    parser.add_argument("-d2", "--dayEnd", help="Day end", type=int, default=dt.datetime.now().day)

    # Print version
    #parser.add_argument("--version", action="version", version='%(prog)s - Version 1.0')

    # Parse argument
    return parser.parse_args()



if __name__ == "__main__":
    main()
    
    
    
    
    
    
    
    