'''
Copyright (c) 2019, Swiss Federal Institute of Technology (ETH Zurich)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

* Neither the name of the copyright holder nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

@date: January 16, 2019
@author: Samuel Weber and Matthias Meyer
'''

def format2iso(df, deployment, vsensor):
    import pandas as pd
    import numpy as np
    import datetime as dt
    
    df_iso = df.copy()
    colnames = pd.read_csv('./metadata/vsensor_metadata/{:s}_{:s}.csv'.format(deployment, vsensor), skiprows=0)
    columns_name = colnames['colname_new'].values
    columns_unit = colnames['unit'].values
    
    columns_old = np.array(df_iso.columns)
    columns_new = columns_old 
    for col, colname in enumerate(columns_old):
        #print(col)
        i = np.where(columns_name == colname)[0][0]
        columns_new[col] = '{:s} {:s}'.format(columns_name[i],columns_unit[i])
        df_iso.columns = columns_new

    
    df_iso.index = df_iso.index.map(lambda x: dt.datetime.strftime(x, '%Y-%m-%dT%H:%M:%SZ'))
    df_iso.index.name = 'time'
    return df_iso


def save_data(df, path_data_file, depo, vsensor, deployment='matterhorn', yearly=False, iso=False, file_type='hdf', float_format=5):
    import os as os
    import numpy as np
    
    os.makedirs(path_data_file, exist_ok=True)
    
    file_name = '{:s}_{:s}'.format(depo, vsensor)
    
    if yearly == False:
        if iso:
            df = format2iso(df, deployment, vsensor)
        if file_type == 'hdf':
            df.to_hdf('{:s}{:s}.h5'.format(path_data_file, file_name), depo, mode='w')
        elif file_type == 'csv':
            df.to_csv('{:s}{:s}.csv'.format(path_data_file, file_name), mode='w', float_format='%.{:d}f'.format(float_format))
        else:
            print('Variable \'file_type={:s}\' is not defined in function \'save_data\'.'.format(file_type))
    elif yearly == True:
        for year in np.unique(df.index.year):
            df_year = df[df.index.year == year]
            if iso:
                df_year = format2iso(df_year, deployment, vsensor)
                
            if file_type == 'hdf':
                df_year.to_hdf('{:s}{:s}_{:d}.h5'.format(path_data_file, file_name, year), depo, mode='w')
            elif file_type == 'csv':
                df_year.to_csv('{:s}{:s}_{:d}.csv'.format(path_data_file, file_name, year), mode='w', float_format='%.{:d}f'.format(float_format))
            else:
                print('Variable \'file_type={:s}\' is not defined in function \'save_data\'.'.format(file_type))
    
    

def load_data(path_data_file, depo, vsensor, year=[], file_type='csv'):
    import pandas as pd
    
    
    if type(year) is list and len(year) == 0:
        file_name = ['{:s}_{:s}'.format(depo, vsensor)]
    elif type(year) is list and len(year) == 1:
        file_name = ['{:s}_{:s}_{:d}'.format(depo, vsensor, year[0])]
    elif type(year) is list and len(year) > 1:
        file_name = []
        for y in year:
            file_name.append('{:s}_{:s}_{:d}'.format(depo, vsensor, y))
    elif type(year) is int:
        file_name = ['{:s}_{:s}_{:d}'.format(depo, vsensor, year)]
    else:
        print('Parameters are not set correctly.')
    
    df_all = pd.DataFrame()
    for fn in file_name:
        df_year = []
        try:
            if file_type == 'hdf':
                df_year = pd.read_hdf('{:s}{:s}.h5'.format(path_data_file, fn), depo)
            elif file_type == 'csv':
                #df_year = pd.read_csv('{:s}{:s}.csv'.format(path_data_file, fn)).set_index('time')
                df_year = pd.read_csv('{:s}{:s}.csv'.format(path_data_file, fn))
                
                print('Hack: Remove this condition, when all data is imported again.')
                if 'Unnamed: 0' in df_year.columns:
                    df_year['time'] = pd.to_datetime(df_year['Unnamed: 0'])
                    df_year.drop(['Unnamed: 0'], axis=1, inplace=True)
                elif 'time' in df_year.columns:
                    df_year['time'] = pd.to_datetime(df_year['time'])
                    
                df_year = df_year.set_index('time')
            else:
                print('Variable \'file_type={:s}\' is not defined in function \'load_data\'.'.format(file_type))
        except:
            print('No data for ' + fn)
            pass
        if len(df_year) > 1:
            df_all = pd.concat([df_all, df_year])
    
    return df_all




def get_GSNdata(deployment, position, vsensor, tbeg, tend):
    """ 
    Get GSNdata with HTTP access
    :param deployment: PermaSense deployment (matterhorn, jungfraujoch, dirruhorn, ?)
    :type deployment: String
    :param position: Position (location) number
    :type position: integer
    :param vsensor: Virtual sensor name (check GSN)
    :type vsensor: String
    :param tbeg tend: Start and end time in UTC (datetime.datetime(year,month,day,hour,minute,second), e.g. datetime.datetime(2015,8,1,0,0,0) '2015-08-01T00:00:00')
    :returns: Timeseries (pandas dataframe)
    
    Problems with plotting 
    http://apache-zeppelin-users-incubating-mailing-list.75479.x6.nabble.com/Issue-with-matplotlib-in-Zeppelin-when-rendering-time-series-plots-td5139.html
    https://issues.apache.org/jira/browse/ZEPPELIN-1904
    """
#    tbeg = TBEG
#    tend = TEND
    import pandas as pd
    import numpy as np
    import datetime as dt
#    timestamp2datetime = np.vectorize(dt.datetime.utcfromtimestamp)

    # 1 - DEFINE VSENSOR-DEPENDENT COLUMNS
    colnames = pd.read_csv('./metadata/vsensor_metadata/{:s}_{:s}.csv'.format(deployment, vsensor), skiprows=0)
    columns_old = colnames['colname_old'].values
    columns_new = colnames['colname_new'].values
    if len(columns_old) != len(columns_new):
        print('WARNING: Length of \'columns_old\' ({:d}) is not equal length of  \'columns_new\' ({:d})'.format(len(columns_old),len(columns_new)))
    
    
    # 2 - DEFINE CONDITIONS AND CREATE HTTP QUERY
    
    # Set server
    server = "http://data.permasense.ch/"
    
    # Create virtual_sensor    
    virtual_sensor = deployment + '_' + vsensor
    
    # Create query and add time as well as position selection
    query= 'vs[1]={:s}&time_format=iso&timeline=generation_time&field[1]=All&from={:s}&to={:s}&c_vs[1]={:s}&c_join[1]=and&c_field[1]=position&c_min[1]={:02d}&c_max[1]={:02d}'.format(virtual_sensor, tbeg.strftime('%d/%m/%Y+%H:%M:%S'), tend.strftime('%d/%m/%Y+%H:%M:%S'), virtual_sensor, int(position)-1, position)
    
    # query extension for images
    if vsensor == 'binary__mapped':
        query = query + '&vs[2]={:s}&field[2]=relative_file&c_join[2]=and&c_vs[2]={:s}&c_field[2]=file_complete&c_min[2]=0&c_max[2]=1&vs[3]={:s}&field[3]=generation_time&c_join[3]=and&c_vs[3]={:s}&c_field[3]=file_size&c_min[3]=2000000&c_max[3]=%2Binf&download_format=csv'.format(virtual_sensor, virtual_sensor, virtual_sensor, virtual_sensor)

    # Construct url:
    url = server  +  "multidata?"  +  query 
    print('The GSN http-query is:\n{:s}'.format(url))


    # 3 - ACCESS DATA AND CREATE PANDAS DATAFRAME
    try:
        d= []
        d = pd.read_csv(url,skiprows=2) # skip header lines (first 2) for import: skiprows=2
        df = pd.DataFrame(columns=columns_new)
        df.time = pd.to_datetime(d.generation_time)
        
        #if depo in ['mh25', 'mh27old', 'mh27new', 'mh30', 'jj22', 'jj25', 'mh-nodehealth']:
        d = d.convert_objects(convert_numeric=True)
#        df = pd.DataFrame(columns=columns_new)
#        df.time = timestamp2datetime(d['generation_time']/1000)
        for i in range(len(columns_old)):
            if columns_new[i] != 'time':
                setattr(df, columns_new[i], getattr(d,columns_old[i]))
            
        df = df.sort_values(by='time')
        
    except:
        df = pd.DataFrame(np.zeros([1,len(columns_new)]), columns=columns_new)
    
    # Remove columns with names 'delete'
    try:
        df.drop(['delete'], axis=1, inplace=True)
    except:
        pass
    
    # Remove columns with only 'null'
    df = df.replace(r'null', np.nan, regex=True)
    isnull = df.isnull().all()
    [df.drop([col_name], axis=1, inplace=True) for col_name in df.columns[isnull]]
    
    df = df.set_index('time')
    
    return df
    

#%%
def filter_data(df_raw, depo):
    '''
    Filter raw data according sensortype-specific reference resistivity measurements
    '''
    import numpy as np
    import pandas as pd
    import datetime as dt
    
    # Load all sensor metadata
    sensor_metadata = pd.read_csv('./metadata/filter_metadata.csv')
    
    # Select position (sensortype) specific reference resistivity values 
    sensor_metadata_sel = sensor_metadata.loc[(sensor_metadata['depo']==depo.lower())] # 
        
    if np.isnan(np.sum(sensor_metadata_sel['ref1'])) or ('ref1' not in df_raw.dtypes):
        print('No reference/calibration values are given...')
        return df_raw
    
    else:
        # Create empty dataframe
        df_filt_all = pd.DataFrame()
        
        # Loop through all sensors 
        for index, row in sensor_metadata_sel.iterrows():
            print(index)
            
            # Define time range
            t1 = row['time_beg']
            t2 = row['time_end']
            if isinstance(t1, str):
                tb = dt.datetime.strptime(t1, '%Y-%m-%d')
            else:
                tb = dt.datetime(2000,1,1)
            if isinstance(t2, str):
                te = dt.datetime.strptime(t2, '%Y-%m-%d')
            else:
                te = dt.datetime(2100,1,1)
                
            # Crop dataframe to valid time range
            df_red = df_raw[(df_raw.index > tb) & (df_raw.index < te)]
            
            # Define lower ('r*low') and upper ('r*high') limit of tolerance
            r1low  = row['ref1'] - row['dref1']
            r1high = row['ref1'] + row['dref1']
            r2low  = row['ref2'] - row['dref2']
            r2high = row['ref2'] + row['dref2']
            r6low  = row['ref6'] - np.float(row['dref6'])
            r6high = row['ref6'] + np.float(row['dref6'])
            
            # Filter according these limits
            if row['sensortype'] in ['sensorrod','moisturechain']:
                df_filt = df_red.loc[(df_red['ref1'] > r1low) & (df_red['ref1'] < r1high ) & (df_red['ref6'] > r6low ) & (df_red['ref6'] < r6high )]
                #df_filt = df_red.loc[(df_red['ref1'] > r1low) & (df_red['ref1'] < r1high ) & (df_red['ref6'] > r6low ) & (df_red['ref6'] < r6high )]
                #df_filt.drop(['ref1'], axis=1, inplace=True)
                #df_filt.drop(['ref6'], axis=1, inplace=True)
            elif row['sensortype'] in ['sensorrod_ETH_6t']:
                df_filt = df_red.loc[(df_red['ref1'] > r1low ) & (df_red['ref1'] < r1high ) & (df_red['ref2'] > r2low ) & (df_red['ref2'] < r2high )]
                #df_filt.drop(['ref1'], axis=1, inplace=True)
                #df_filt.drop(['ref2'], axis=1, inplace=True)
            elif row['sensortype'] == 'thermistorchain': # only one reference value is available
                df_filt = df_red.loc[(df_red['ref1'] > r1low ) & (df_red['ref1'] < r1high )]
                #df_filt.drop(['ref1'], axis=1, inplace=True)
            elif row['sensortype'] == 'sensorrod_UMS_th3': # no reference value is available
                df_filt = df_red.copy()
                
            if len(df_filt) > 1:
                df_filt_all = pd.concat([df_filt_all, df_filt])
        
        for colname in ['temperature_nearsurface_t2', 'temperature_nearsurface_t1']:
            if colname in df_raw.columns and ~df_raw[colname].isnull().all():
                df_sel = pd.DataFrame(df_raw[colname][~df_raw[colname].isnull()], columns=[colname])
                df_filt_all = df_sel.append(df_filt_all)
                # change order of columns
                cols = df_filt_all.columns.tolist()
                cols = cols[-1:] + cols[:-1]
                df_filt_all = df_filt_all[cols]  #    OR    df_filt_all = df_filt_all.ix[:, cols]
        
        return df_filt_all


def clean_data(df_raw, depo, vsensor):
    '''
    Filter data according manual interaction.
    '''
    import numbers
    import numpy as np
    import pandas as pd
    import datetime as dt
    
    df_cleaned = df_raw.copy()
    
    clean_list = pd.read_csv('./metadata/clean_metadata.csv')
    clean_depo = clean_list.loc[(clean_list.depo==depo.lower()) & (clean_list.vsensor==vsensor)]
    
    for i in clean_depo.index:
        clean_sel = clean_depo[clean_depo.index==i]
        
        # Define time range
        if isinstance(clean_sel['time_beg'].values[0], str):
            tb = dt.datetime.strptime(clean_sel['time_beg'].values[0], '%Y-%m-%d')
        else:
            tb = dt.datetime(2000,1,1)
        if isinstance(clean_sel['time_end'].values[0], str):
            te = dt.datetime.strptime(clean_sel['time_end'].values[0], '%Y-%m-%d')
        else:
            te = dt.datetime(2100,1,1)
        
        # Index of time condition
        ind_time = (df_cleaned.index > tb) & (df_cleaned.index < te)
            
        # Index of condition
        if clean_sel['var'].values[0] == '*':
            ind_cond = np.ones(len(ind_time), dtype=bool)
        elif isinstance(clean_sel['cond_op'].values[0], str) and isinstance(clean_sel['cond_par'].values[0], numbers.Number):
            if clean_sel['cond_op'].values[0] == '>':
                ind_cond = (df_cleaned[clean_sel['var'].values[0]] > np.float(clean_sel['cond_par'].values[0]))
            elif clean_sel['cond_op'].values[0] == '=':
                ind_cond = (df_cleaned[clean_sel['var'].values[0]] == np.float(clean_sel['cond_par'].values[0]))
            elif clean_sel['cond_op'].values[0] == '<':
                ind_cond = (df_cleaned[clean_sel['var'].values[0]] < np.float(clean_sel['cond_par'].values[0]))
            ind_cond = ind_cond.values
        else:
            ind_cond = np.ones(len(ind_time), dtype=bool)
        
        # Apply operation
        ind = ind_time * ind_cond
        if clean_sel['var'].values[0] == '*':
            if clean_sel['op'].values[0] == 'ok':
                print('No cleaning...')
            elif clean_sel['op'].values[0] == 'del':
                df_cleaned.drop(df_cleaned.index[ind], inplace=True)
            else:
                print('The combination of var=\'*\' and op=\'{:s}\' is not valid (depo=\'{:s}\', vsensor=\'{:s}\').'.format(clean_sel['op'].values[0], depo, vsensor))
        else:
            if clean_sel['op'].values[0] == 'ok':
                print('No cleaning...')
            elif clean_sel['op'].values[0] == 'del':
                #print('delete some values')
                df_cleaned.loc[ind, clean_sel['var'].values[0]] = df_cleaned.loc[ind, clean_sel['var'].values[0]] + np.NaN
            elif clean_sel['op'].values[0] == 'offset':
                df_cleaned.loc[ind, clean_sel['var'].values[0]] = df_cleaned.loc[ind, clean_sel['var'].values[0]] + clean_sel['par'].values[0]
            elif clean_sel['op'].values[0] == 'mult':
                df_cleaned.loc[ind, clean_sel['var'].values[0]] = df_cleaned.loc[ind, clean_sel['var'].values[0]] * clean_sel['par'].values[0]
            elif clean_sel['op'].values[0] == 'replace':
                df_cleaned.loc[ind, clean_sel['var'].values[0]] = df_cleaned.loc[ind, clean_sel['var'].values[0]] * 0 + clean_sel['par'].values[0]
        
    return df_cleaned


def get_GSNimg(path_img, deployment, position, vsensor, tbeg, tend, n_img=10):
    import os as os
    import numpy as np
    import pandas as pd
    import urllib.request
    import subprocess, shlex
    import piexif
    
    if isinstance(n_img, int):
        pass
    elif isinstance(n_img, str) and n_img=='ALL':
        n_img = np.inf
    else:
        print('\"n_img\" not properly defined: \"n_img\" must be an integer value or \"ALL\"')
        print('ADD WARNING')
    
   
    df = get_GSNdata(deployment, position, vsensor, tbeg, tend)
    df['pk'] = df.data.str.split('=', expand=True)[3]
    df['fname'] = df.relative_file.str.split('/', expand=True)[2].str.split('.', expand=True)[0]
    df['time'] = df.relative_file.str.split('/', expand=True)[2].str.split('.', expand=True)[0]
    df = df.drop(df.index[df.time.apply(len)!=15])
    df.time = pd.to_datetime(df.time, format='%Y%m%d_%H%M%S')
    df = df.set_index('time')
    
    count = 0
    for index, row in df.iterrows():
        count += 1
        if count > n_img:
            if n_img == 10:
                print('Downloading and converting images is very time consuming. Therefore, this operation is (optionally) limited to the first 10 images.')
            break
        
        # Create subdirectory
        subfolder = '{:s}-{:s}-{:s}'.format(row.fname.split('_')[0][0:4], row.fname.split('_')[0][4:6], row.fname.split('_')[0][6:])
        os.makedirs(path_img + subfolder, exist_ok=True)
        
        # Path and filename definitions
        src = '{:s}{:s}/{:s}.NEF'.format(path_img,subfolder,row.fname)
        dst = '{:s}{:s}/{:s}.JPG'.format(path_img,subfolder,row.fname)
        
        # Query images, convert to .JPG and store
        urllib.request.urlretrieve('http://data.permasense.ch/field?vs=matterhorn_binary__mapped&field=data&pk={:s}'.format(row.pk[0]), src)
        try:
            cmd = shlex.split('./NConvert/nconvert -quiet -out jpeg -embedded_jpeg -overwrite -o %s %s' % (dst,src))
        except:
            print('Do you have downloaded the NConvert (batch image processor) and copied it into the folder \"gsn_data_management_bin\"?')
            pass
        proc_util = subprocess.run(cmd, stdout=subprocess.PIPE)
        cmd = shlex.split('./NConvert/nconvert -quiet -out jpeg -jpegtrans rot90 -overwrite -o %s %s' % (dst,dst))
        proc_util = subprocess.run(cmd, stdout=subprocess.PIPE)
        exif_dict = piexif.load(dst)
        exif_dict["0th"][piexif.ImageIFD.Orientation] = 8
        piexif.insert(piexif.dump(exif_dict),dst)
        os.remove(src)


#
#def run_dataavailability_check_MH(tbeg=0,tend=0):
#    '''
#    % Run in command line
#    % ssh ims@whymper.ethz.ch
#    % cd bin/
#    % ./check_mh19_miniseed_availability.sh
#    % ./check_mh36_miniseed_availability.sh
#    % ./check_mh38_miniseed_availability.sh
#    % ./check_mh40_miniseed_availability.sh
#    % ./check_mh44_miniseed_availability.sh
#    % exit
#    % scp ims@whymper.ethz.ch:bin/MH*_miniseed_availability.out /home/webersam/Data/MH/dataMSAE/timeseries/dataMSavailability/
#    % scp ims@whymper.ethz.ch:bin/MH19_pictures_availability.out /home/webersam/Data/MH/dataMSAE/timeseries/dataMSavailability/
#    % scp /home/webersam/Data/MH/dataMSAE/timeseries/dataMSavailability/MH*_miniseed_availability.out whymper.ethz.ch:/usr/whymper/data-tik-02/permasense_vault/data_archive/deployments/matterhorn2007/analysis_AEMS/timeseries/dataMSavailability/
#    % scp /home/webersam/Data/MH/dataMSAE/timeseries/dataMSavailability/MH19_pictures_availability.out whymper.ethz.ch:/usr/whymper/data-tik-02/permasense_vault/data_archive/deployments/matterhorn2007/analysis_AEMS/timeseries/dataMSavailability/
#    '''
#    from toolbox import  get_source_path
#    PATH_DATA_SOURCE = get_source_path() 
#    import pandas as pd
#    import datetime as dt
#    from tqdm import tqdm
#    import numpy as np
#    
#    if tbeg == 0:
#        tbeg = dt.datetime(2008,1,1)
#    if tend == 0:
#        tend = dt.datetime.utcnow()
#    reindex_range = pd.date_range(tbeg, tend, freq='1h', utc=True)
#
#    NH = import_GSNdata('mh-nodehealth',tbeg,tend)
#    
#    for position in [1,2,3,4,5,6,7,8,9,10,11,12,15,18,20,21,22,25,27,30,33,34,35,40,42,46,47]:
#        tmp = NH.loc[NH.position==position,:]
#        index = pd.to_datetime(pd.DatetimeIndex(tmp.index),utc=True)
#        available = np.array(index,dtype=bool)
#        df = pd.DataFrame({'MH{:02d}'.format(position):available},index=index)
#        df = timeseries_aggregation(df,aggregation_interval=60,function='sum')
#        df = df.reindex(reindex_range)
#        ind = pd.isnull(df['MH{:02d}'.format(position)])
#        df['MH{:02d}'.format(position)][ind] = np.array(ind[ind==True]*0,dtype=bool)
#        df = df.astype('bool')
#        exec('MH{:02d} = df'.format(position))
#        
#    MH29high = df.copy()
#    MH29high.rename(columns={'MH29': 'MH29high'}, inplace=True)
#    MH29low = df.copy()
#    MH29low.rename(columns={'MH29': 'MH29low'}, inplace=True)
#    
#    for position in [36,38,40]:
#        tmp = pd.read_csv(PATH_DATA_SOURCE + 'timeseries/dataMSavailability/MH{:d}_miniseed_availability.out'.format(position), sep='/',header=None)
#        df = pd.DataFrame(index=range(len(tmp)),columns={'time','MH{:d}'.format(position)})
#        for i in tqdm(range(len(tmp))):
#            if position in [36,38]:
#                try:
#                    year = int(tmp[11][i][16:20])
#                    month = int(tmp[11][i][20:22])
#                    day = int(tmp[11][i][22:24])
#                    hour = int(tmp[11][i][25:27])
#                    df['time'][i] = dt.datetime(year,month,day,hour)
#                    df['MH{:d}'.format(position)][i] = 1
#                except:
#                    pass
#            elif position in [40]:
#                try:
#                    year = int(tmp[4][i][16:20])
#                    month = int(tmp[4][i][20:22])
#                    day = int(tmp[4][i][22:24])
#                    hour = int(tmp[4][i][25:27])
#                    df['time'][i] = dt.datetime(year,month,day,hour)
#                    df['MH{:d}'.format(position)][i] = 1
#                except:
#                    pass
#        df.set_index('time',inplace=True)
#        df = timeseries_aggregation(df,aggregation_interval=60,function='sum')
#        df = df.reindex(reindex_range)
#        ind = pd.isnull(df['MH{:d}'.format(position)])
#        df['MH{:d}'.format(position)][ind] = np.array(ind[ind==True]*0,dtype=bool)
#        df = df.astype('bool')
#        exec('MH{:d} = df'.format(position))
#    
#    dataavailability = pd.concat([MH01,MH02,MH03,MH04,MH05,MH06,MH07,MH08,MH09,MH10,MH11,MH12,MH15,MH18,MH20,MH21,MH22,MH25,MH27,MH30,MH33,MH34,MH35,MH40,MH42,MH46,MH47,MH29high,MH29low,MH36,MH38,MH40],axis=1)
#    dataavailability.to_hdf(PATH_DATA_SOURCE + 'timeseries/dataavailability.h5','dataavailability', mode='w')
#
#def get_disturbance_dataavailability_MH(reindex_range,rundataavailabilitycheck=False):
#    '''
#    Get timeseries of disturbance (fieldvisit, mountaineers) and data availability
#    :param reindex_range: pd.date_range(dt.datetime(), dt.datetime(), freq='1min', utc=True)
#    :returns: disturbance (1min interval) and dataavailability (1H interval) timeseries (pandas dataframe)
#
#    '''
#    import numpy as np
#    import pandas as pd
#    import datetime as dt
#    from toolbox import  get_source_path
#    PATH_DATA_SOURCE = get_source_path() 
#
#    index_fieldvisit = np.zeros(len(reindex_range))
#    index_mountaineers = index_fieldvisit.copy()
#    
#    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#    # field visit periods
#    windows = [pd.date_range(dt.datetime(2014,8,14), dt.datetime(2014,8,17), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2014,9,3), dt.datetime(2014,9,4), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,1,8), dt.datetime(2015,1,9), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,4,8), dt.datetime(2015,4,9), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,5,28), dt.datetime(2015,5,30), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,6,2), dt.datetime(2015,6,4), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,6,24), dt.datetime(2015,6,26), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,6,27), dt.datetime(2015,6,28), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,10,8), dt.datetime(2015,10,9), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2016,1,21), dt.datetime(2016,1,22), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2016,7,26), dt.datetime(2016,7,28), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2016,7,29), dt.datetime(2016,7,30), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2016,12,1), dt.datetime(2016,12,2), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2017,7,18), dt.datetime(2017,7,20), freq='1min', utc=True)
#               ]
#    timestamps =  pd.date_range(dt.datetime(2014,1,1), dt.datetime(2014,1,1), freq='1min', utc=True) # dummy date, that the timestamps of 'windows' can be appended...
#    for i in range(len(windows)):
#        timestamps = timestamps.append(windows[i])
#
#    for i in range(len(timestamps)):
#        index_fieldvisit[reindex_range == timestamps[i]] = 1
#        
#    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#    # mountaineer activity
#    windows = [pd.date_range(dt.datetime(2015,10,8,8), dt.datetime(2015,10,8,12), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,10,22,16), dt.datetime(2015,10,22,18), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,10,23), dt.datetime(2015,10,23,6), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,10,23,16), dt.datetime(2015,10,23,18), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,10,24,6), dt.datetime(2015,10,24,8), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,10,25,10), dt.datetime(2015,10,25,10), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,10,25,12), dt.datetime(2015,10,25,14), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,10,26,12), dt.datetime(2015,10,26,12), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,10,26,16), dt.datetime(2015,10,26,16), freq='1min', utc=True),
#               
#               pd.date_range(dt.datetime(2015,11,5,6), dt.datetime(2015,11,5,6), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,11,5,14), dt.datetime(2015,11,5,16), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,11,8,8), dt.datetime(2015,11,8,10), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,11,9,6), dt.datetime(2015,11,9,8), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,11,11,10), dt.datetime(2015,11,11,10), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,11,11,16), dt.datetime(2015,11,11,16), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,11,13,14), dt.datetime(2015,11,13,14), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,11,15,10), dt.datetime(2015,11,15,10), freq='1min', utc=True),
#               pd.date_range(dt.datetime(2015,11,16,2), dt.datetime(2015,11,16,2), freq='1min', utc=True),
#               
#               pd.date_range(dt.datetime(2015,12,30,4), dt.datetime(2015,12,30,6), freq='1min', utc=True)
#               ]
#    timestamps =  pd.date_range(dt.datetime(2014,1,1), dt.datetime(2014,1,1), freq='1min', utc=True) # dummy date, that the timestamps of 'windows' can be appended...
#    for i in range(len(windows)):
#        timestamps = timestamps.append(windows[i])
#
#    for i in range(len(timestamps)):
#        index_mountaineers[reindex_range == timestamps[i]] = 1
#        
#    if rundataavailabilitycheck == True:
#        run_dataavailability_check_MH()
#    else:
#        print('Run load_alldata.run_dataavailability_check_MH() for an update:')
#        print('from load_alldata import run_dataavailability_check_MH')
#        print('run_dataavailability_check_MH()')
#        print('>get_disturbance_dataavailability_MH(reindex_range,rundataavailabilitycheck=False)')
#    dataavailability = pd.read_hdf(PATH_DATA_SOURCE + 'timeseries/dataavailability.h5','dataavailability')
#    
#    dataavailability['MH3638'] = np.array(dataavailability['MH36'] * dataavailability['MH38'],dtype=bool)
#    dataavailability = dataavailability.reindex(reindex_range).fillna(method='ffill')
#    
#    # Manual changes
#    dataavailability.MH27[(dataavailability.index > dt.datetime(2017, 6, 26, 16)) & (dataavailability.index < dt.datetime(2017, 7, 21))] = False
#    dataavailability.MH29high[(dataavailability.index > dt.datetime(2016,1,21)) & (dataavailability.index < dt.datetime(2016,7,10))] = False
#    dataavailability.MH29low[(dataavailability.index > dt.datetime(2016,1,21)) & (dataavailability.index < dt.datetime(2016,7,10))] = False
#
#    
#    disturbance = pd.DataFrame({'fieldvisit':np.array(index_fieldvisit,dtype=bool),'mountaineers':np.array(index_mountaineers,dtype=bool)},index=reindex_range)
#    return disturbance, dataavailability